/*************************************************************************
Autor:    Frank Nitzsche beta-frank@beta-x.de www.beta-x.de
Datei:    rtc.c
Version:  1.1 - 09.Jan.2007
Compiler: 4.1.2 (WinAVR 20070525)
**************************************************************************/
#include "pcf8583.h"
#include "i2cmaster.h"

#define  BIN2BCD(b)  (((b)%10) | (((b)/10)<<4))
#define  BCD2BIN(b)  (((b)&0xf) + ((b)>>4)*10)

#if(PCF8583_MASK_WDAY_AND_YEAR)
   #define PCF8583_MASK 0x08
#else
   #define PCF8583_MASK 0x00
#endif

/*************************************************************************/
static void writeByteRtc(uint8_t Adr, uint8_t Val){
/*************************************************************************/
   i2c_start(PCF8583_WR_ADR);
   i2c_write(Adr);
   i2c_write(Val);
   i2c_stop();
}

/*************************************************************************/
static uint8_t readByteRtc(uint8_t Adr){
/*************************************************************************/
   uint8_t retval;
   i2c_start(PCF8583_WR_ADR);
   i2c_write(Adr);
   i2c_rep_start(PCF8583_RD_ADR);
   retval = i2c_readNak();
   i2c_stop();
   return(retval);    
}


/*************************************************************************/
void initRtc(void){
/*************************************************************************/
   i2c_start(PCF8583_WR_ADR);
   i2c_write(PCF8583_CTRL_STATUS_REG);
   i2c_write(PCF8583_MASK);  //Set (or not) weekday- & year-bits-mask
   i2c_stop();
}

/*************************************************************************/
void resetRtc(void){
/*************************************************************************/
   DATETIME_T MyDT;
   MyDT.day=1;
   MyDT.wday=0;
   MyDT.mon=1;
   MyDT.year=07;
   MyDT.hour=8;
   MyDT.min=45;
   MyDT.sec=0;
   writeRtc(&MyDT);
}

/*************************************************************************/
void writeRtc(DATETIME_T *DT){
/*************************************************************************/
   i2c_start(PCF8583_WR_ADR);
   i2c_write(PCF8583_CTRL_STATUS_REG);
   i2c_write(PCF8583_STOP_COUNTING | PCF8583_MASK);  //Stop counting
   //Time
   i2c_write(0x00);  //1/100 Seconds
   i2c_write(BIN2BCD(DT->sec));
   i2c_write(BIN2BCD(DT->min));
   i2c_write(BIN2BCD(DT->hour));
   //Date
   i2c_write(BIN2BCD(DT->day) | (DT->year << 6));
   i2c_write(BIN2BCD(DT->mon) | (DT->wday << 5));
   i2c_stop();

   writeByteRtc(PCF8583_YEAR_REG, DT->year);

   initRtc();        //Enable counting
}

/*************************************************************************/
void readRtc(DATETIME_T *DT){
/*************************************************************************/
   uint8_t Buf;
   i2c_start(PCF8583_WR_ADR);
   i2c_write(PCF8583_SECONDS_REG);
   i2c_rep_start(PCF8583_RD_ADR);
   //Time
   DT->sec  = i2c_readAck();
   DT->sec  = BCD2BIN(DT->sec);
   DT->min  = i2c_readAck();
   DT->min  = BCD2BIN(DT->min);
   DT->hour = i2c_readAck();
   DT->hour = BCD2BIN(DT->hour&0x3f);
   //Date
   Buf      = i2c_readAck();
   DT->year = Buf>>6;
   DT->day  = BCD2BIN(Buf&0x3f);
   Buf      = i2c_readNak();
   DT->mon  = BCD2BIN(Buf&0x1f);
   DT->wday = Buf>>5;
   i2c_stop();   
   // Read the year byte from NVRAM.
   Buf = readByteRtc(PCF8583_YEAR_REG);
   // Check if the two "year bits" were incremented by
   // the PCF8583.  If so, increment the 8-bit year
   // byte (read from NVRAM) by the same amount. 
   while(DT->year != (Buf & 3))
      Buf++;
   DT->year = Buf;
   // Now update the year byte in the NVRAM
   // inside the PCF8583.
   writeByteRtc(PCF8583_YEAR_REG, Buf); 
}



