/*************************************************************************
Autor:    Frank Nitzsche beta-frank@beta-x.de www.beta-x.de
Datei:    rtc.h
Version:  1.1 - 09.Jan.2007
Compiler: GCC 4.1.2 (WinAVR 20070525)
**************************************************************************/
#ifndef PCF8583_DEF
#define PCF8583_DEF
#include <avr/io.h>

// Set to 1 if Pin A0 is tied to Vcc
#define PCF8583_A0_VCC 0

// Set to 1 for masking out the year and weekday bits.
#define PCF8583_MASK_WDAY_AND_YEAR 0

#if (PCF8583_A0_VCC)
   #define PCF8583_RD_ADR 0xA3      // Slave address read PCF8583
   #define PCF8583_WR_ADR 0xA2      // Slave address write PCF8583
#else
   #define PCF8583_RD_ADR 0xA1      // Slave address read PCF8583
   #define PCF8583_WR_ADR 0xA0      // Slave address write PCF8583
#endif

// Register addresses
#define PCF8583_CTRL_STATUS_REG    0x00
#define PCF8583_100S_REG           0x01
#define PCF8583_SECONDS_REG        0x02
#define PCF8583_MINUTES_REG        0x03
#define PCF8583_HOURS_REG          0x04
#define PCF8583_DATE_REG           0x05
#define PCF8583_MONTHS_REG         0x06
#define PCF8583_TIMER_REG          0x07

#define PCF8583_ALARM_CONTROL_REG  0x08
#define PCF8583_ALARM_100S_REG     0x09
#define PCF8583_ALARM_SECS_REG     0x0A
#define PCF8583_ALARM_MINS_REG     0x0B
#define PCF8583_ALARM_HOURS_REG    0x0C
#define PCF8583_ALARM_DATE_REG     0x0D
#define PCF8583_ALARM_MONTHS_REG   0x0E
#define PCF8583_ALARM_TIMER_REG    0x0F

// Use the first NVRAM address for the year byte.
#define PCF8583_YEAR_REG           0x10

// Commands for the Control/Status register.
#define PCF8583_START_COUNTING     0x00
#define PCF8583_STOP_COUNTING      0x80

// struct for holding date/time
typedef struct{ 
   uint8_t sec;
   uint8_t min;
   uint8_t hour; 
   uint8_t day;   
   uint8_t wday;
   uint8_t mon;
   uint8_t year;
}DATETIME_T;

/*************************************************************************/
void initRtc(void);
/*************************************************************************/
void readRtc(DATETIME_T *DT);
/*************************************************************************/
void writeRtc(DATETIME_T *DT);
/*************************************************************************/
void resetRtc(void);
/*************************************************************************/

#endif /*PCF8583_DEF*/
